#include <assert.h>
#include <math.h>
#include <mpi.h>
#include <omp.h>
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char *argv[])
{
  int numprocs, rank, namelen;
  char processor_name[MPI_MAX_PROCESSOR_NAME];
  int iam = 0, np = 1;

  double t1, t2, tick;
  double omp_start,omp_end,omp_wtick;

  MPI_Init(&argc, &argv);
  MPI_Comm_size(MPI_COMM_WORLD, &numprocs);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Get_processor_name(processor_name, &namelen);

/* MPI time measurement start */
  MPI_Barrier(MPI_COMM_WORLD); /* IMPORTANT */
  t1 = MPI_Wtime();

  #pragma omp parallel default(shared) private(iam, np)
  {
    np = omp_get_num_threads();
    iam = omp_get_thread_num();
//    printf("thread %d | num_threads %d | MPI process %d | num MPI proc %d | processor_name %s\n",
//           iam, np, rank, numprocs, processor_name);

/* OMP time measurement start */
    #pragma omp barrier
    #pragma omp master 
    omp_start = omp_get_wtime( );

    sleep(1+rank);

    omp_end = omp_get_wtime( );
    #pragma omp barrier
/* OMP time measurement end   */

    tick = MPI_Wtick();
    omp_wtick = omp_get_wtick( );

/* use time on master node */
    if(rank == 0 && iam ==0)
    {
        printf("MPI processes:        number=%d\n",numprocs);
        printf("OMP processes:       threads=%d\n\n",np);

        printf("A single tick\n\t\tMPI\t\ttick = %.3e s\t1/tick = %.3e /s\n",
                  tick, 1/tick);
        printf("\t\tOMP\t\ttick = %.3e s\t1/tick = %.3e /s\n",
                  omp_wtick, 1.0 / omp_wtick);

        fflush(stdout);
    }

    printf("process %d / thread  %d: OMP_Wtime %1.6f s\n", rank, iam, omp_end - omp_start);
    fflush(stdout);

  }
  MPI_Barrier(MPI_COMM_WORLD); /* IMPORTANT */
  t2 = MPI_Wtime();
/* MPI time measurement end   */  
  printf("process %d               MPI_Wtime %1.6f s \n", rank, t2-t1);

  MPI_Finalize();
  return 0;
}

